@@ -4,6 +4,8 @@
  * Copyright (c) 2010-2019, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
+const Config = imports.misc.config;
+
 const Gettext = imports.gettext;
 
 const { GPaste } = imports.gi;
@@ -12,9 +14,9 @@ const ExtensionUtils = imports.misc.extensionUtils;
 const Me = ExtensionUtils.getCurrentExtension();
 
 function init() {
-    const metadata = Me.metadata;
-    Gettext.bindtextdomain(metadata.gettext_package, metadata.localedir);
-    Gettext.textdomain(metadata.gettext_package);
+    const domain = Me.metadata['gettext-domain'];
+    Gettext.bindtextdomain(domain, Config.LOCALEDIR);
+    Gettext.textdomain(domain);
 }
 
 function buildPrefsWidget() {
