@@ -77,6 +77,7 @@ g_paste_file_backend_write_history_file (const GPasteStorageBackend *self,
     {
         GPasteItem *item = history->data;
         const gchar *kind = g_paste_item_get_kind (item);
+        const gchar *uuid = g_paste_item_get_uuid (item);
 
         if (g_paste_str_equal (kind, "Password"))
             continue;
@@ -86,6 +87,8 @@ g_paste_file_backend_write_history_file (const GPasteStorageBackend *self,
 
         if (!g_output_stream_write_all (stream, "  <item kind=\"", 14, NULL, NULL /* cancellable */, NULL /* error */) ||
             !g_output_stream_write_all (stream, kind, strlen (kind), NULL, NULL /* cancellable */, NULL /* error */) ||
+            !g_output_stream_write_all (stream, "\" uuid=\"", 8, NULL, NULL /* cancellable */, NULL /* error */) ||
+            !g_output_stream_write_all (stream, uuid, strlen (uuid), NULL, NULL /* cancellable */, NULL /* error */) ||
             (_G_PASTE_IS_IMAGE_ITEM (item) && !_g_paste_file_backend_write_image_metadata (stream, _G_PASTE_IMAGE_ITEM (item))) ||
             !g_output_stream_write_all (stream, "\">\n    <value><![CDATA[", 23, NULL, NULL /*cancellable */, NULL /*error */) ||
             !g_output_stream_write_all (stream, text, strlen (text), NULL, NULL /* cancellable */, NULL /* error */) ||
@@ -145,6 +148,7 @@ typedef struct
     guint64           current_size;
     guint64           max_size;
     gboolean          images_support;
+    gchar            *uuid;
     gchar            *date;
     gchar            *name;
     gchar            *text;
@@ -163,6 +167,21 @@ typedef struct
     ASSERT_STATE (x);      \
     data->state = y
 
+static gboolean
+history_contains_uuid (const GList *history,
+                       const gchar *uuid)
+{
+    for (; history; history = g_list_next (history))
+    {
+        const GPasteItem *item = history->data;
+
+        if (g_paste_str_equal (g_paste_item_get_uuid (item), uuid))
+            return TRUE;
+    }
+
+    return FALSE;
+}
+
 static void
 start_tag (GMarkupParseContext *context G_GNUC_UNUSED,
            const gchar         *element_name,
@@ -199,6 +218,7 @@ start_tag (GMarkupParseContext *context G_GNUC_UNUSED,
     else if (g_paste_str_equal (element_name, "item"))
     {
         SWITCH_STATE (IN_HISTORY, IN_ITEM);
+        g_clear_pointer (&data->uuid, g_free);
         g_clear_pointer (&data->date, g_free);
         g_clear_pointer (&data->name, g_free);
         g_clear_pointer (&data->text, g_free);
@@ -217,6 +237,11 @@ start_tag (GMarkupParseContext *context G_GNUC_UNUSED,
                 else
                     g_warning ("Unknown item kind: %s", *v);
             }
+            else if (g_paste_str_equal (*a, "uuid"))
+            {
+                if (g_uuid_string_is_valid (*v) && !history_contains_uuid (data->history, *v))
+                    data->uuid = g_strdup (*v);
+            }
             else if (g_paste_str_equal (*a, "date"))
             {
                 if (data->type != IMAGE)
@@ -304,6 +329,10 @@ add_item (Data *data)
 
     if (item)
     {
+        if (!data->uuid)
+            data->uuid = g_uuid_string_random ();
+
+        g_paste_item_set_uuid (item, data->uuid);
         data->mem_size += g_paste_item_get_size (item);
         data->history = g_list_append (data->history, item);
         ++data->current_size;;
@@ -481,6 +510,7 @@ g_paste_file_backend_read_history_file (const GPasteStorageBackend *self,
             NULL,
             NULL,
             NULL,
+            NULL,
             HISTORY_INVALID,
             G_PASTE_SPECIAL_ATOM_INVALID
         };
