@@ -19,11 +19,15 @@ G_BEGIN_DECLS
 
 G_PASTE_FINAL_TYPE (UiItem, ui_item, UI_ITEM, GPasteUiItemSkeleton)
 
-gboolean  g_paste_ui_item_activate  (GPasteUiItem *self);
-void      g_paste_ui_item_refresh   (GPasteUiItem *self);
+gboolean  g_paste_ui_item_activate (GPasteUiItem *self);
+void      g_paste_ui_item_refresh  (GPasteUiItem *self);
+
 void      g_paste_ui_item_set_index (GPasteUiItem *self,
                                      guint64       index);
 
+void      g_paste_ui_item_set_uuid (GPasteUiItem *self,
+                                    const gchar  *uuid);
+
 GtkWidget *g_paste_ui_item_new (GPasteClient   *client,
                                 GPasteSettings *settings,
                                 GtkWindow      *rootwin,
